/*  brlkey
    Copyright(C) 2008 PaulLiu(Ying-Chun Liu). All rights reserved.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include <config.h>

#include <iostream>

#include <cstdio>
#include <cstdlib>
#include <cstring>
#include <ctime>

#include <termios.h>
#include <unistd.h>
#include <pty.h>
#include <signal.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <errno.h>

#include <linux/input.h>

#include "common.h"

using namespace std;

char scancode[KEYARRAYSIZE][50];

int
reverse_scancode (const char *s1)
/* reverse scancode string to scancode */
{
  int i;
  for (i = 0; i < KEYARRAYSIZE; i++)
    {
      if (strcmp (s1, scancode[i]) == 0)
	{
	  return i;
	}
    }
  return -1;
}

int
genarray (const char *s1, int *keyboard_array)
/* string -> scancode array */
{
  char com1[100];
  int i, r1, ret = 0;
  memset (keyboard_array, 0, sizeof (int) * KEYARRAYSIZE);
  while (*s1 != '\0')
    {
      com1[0] = '\0';
      if (*s1 == '<')
	{
	  i = 0;
	  while (*s1 != '>' && *s1 != '\0')
	    {
	      com1[i] = *s1;
	      com1[i + 1] = '\0';
	      i++;
	      s1++;
	    }
	  if (*s1 == '>')
	    {
	      com1[i] = *s1;
	      com1[i + 1] = '\0';
	      i++;
	      s1++;
	    }
	}
      else
	{
	  com1[0] = *s1;
	  com1[1] = '\0';
	  s1++;
	}
      r1 = reverse_scancode (com1);
      if (r1 >= 0)
	{
	  keyboard_array[r1] = 1;
	  ret++;
	}
    }
  return ret;
}

int
readscancode (void)
/* load scancode table */
{
  FILE *file1;
  char filename[1000];
  int com1, i;
  char com2[50];
  for (i = 0; i < KEYARRAYSIZE; i++)
    {
      scancode[i][0] = '\0';
    }
  strcpy (filename, PKGDATADIR);
  strcat (filename, "/");
  strcat (filename, "scancode.txt");
  file1 = fopen (filename, "r");
  if (file1 == NULL)
    {
      printf ("Error: cannot open %s\n", filename);
      return -1;
    }
  while (!feof (file1))
    {
      if (fscanf (file1, "%d%s", &com1, com2) != 2)
	break;
      strcpy (scancode[com1], com2);
    }
  fclose (file1);
  return 0;
}

int
event_handler (enum inputmode mode, const struct keyevent *ev)
{
  switch (mode)
    {
    case INPUTMODE_ISO8859_1:
      {
	iso8859_event_handler (ev);
	break;
      }
    case INPUTMODE_BIG5:
      {
	big5_event_handler (ev);
	break;
      }
    case INPUTMODE_LIU:
      {
	liu_event_handler (ev);
	break;
      }
    case INPUTMODE_NORMAL:
      {
	 int now=(*ev).key;
	   switch((*ev).type)
	   {
	     case KEYEVENT_RELEASE:
	     case KEYEVENT_PRESS:
		{
		  struct input_event event = {};
		  event.type = EV_KEY;
		  event.code = now;
		  event.value = (*ev).type == KEYEVENT_PRESS;
		  if (normalhandler(ev) == 0)
		    output_event(&event);
		}
	   }
      }
      break;
    default:
      {
	cerr << "Error: event_handler() unknown event" << endl;
	break;
      }
    }
  return 0;
}

int
iohandler (void)
/* io handler */
{
  int keyboard_count = 0, keyboard_array[KEYARRAYSIZE];
  int keyboard_array_max[KEYARRAYSIZE], keyboard_count_max = 0;
  enum inputmode im = INPUTMODE_ISO8859_1;
  enum inputmode imflag = INPUTMODE_ISO8859_1;
  struct keyevent ev1;
  struct input_event event;

  memset (keyboard_array, 0, sizeof (keyboard_array));
  keyboard_count_max = 0;
  keyboard_count = 0;
      while (1 == 1)
	{
	  if (input_event(&event) <= 0)	/* read event */
	    {
	      printf("input_event: %s\n",strerror(errno));
	      break;
	    }
	  if (event.type != EV_KEY)
	  {
	    /* not a key press, transmit it. */
	    if (output_event(&event) <= 0) {
	      printf("output_event: %s\n",strerror(errno));
	      break;
	    }
	    continue;
	  }
	  /* update the value of keyboard_array */
	  if (event.value == 0)
	    {
	      if (keyboard_array[event.code] == 1)
		keyboard_count--;
	      keyboard_array[event.code] = 0;
	    }
	  else
	    {
	      if (keyboard_array[event.code] == 0)
		keyboard_count++;
	      keyboard_array[event.code] = 1;
	    }
	  /* update the value of keyboard_count_max */
	  if (keyboard_count > keyboard_count_max)
	    {
	      keyboard_count_max = keyboard_count;
	      memcpy (keyboard_array_max, keyboard_array,
		      sizeof (keyboard_array_max));
	    }

	  /* press Microsoft Window Key -> force quit */
	  if (event.code == KEY_LEFTMETA)
	    {
	      /* Only change mode if windows key is alone */
	      if (event.value && keyboard_count == 1) {
		if(im!=INPUTMODE_NORMAL)
		{
		  im=INPUTMODE_NORMAL;
		  playsound("toenglish");
		}
		else
		{ 
		  im=INPUTMODE_ISO8859_1;
		  playsound("tobig5");
		}
	      }
	      continue;
	    }
	  /* send key press/release event */
	  memset (&ev1, 0, sizeof (ev1));
	  memcpy (ev1.keyarray, keyboard_array, sizeof (ev1.keyarray));
	  ev1.keyarray_count = keyboard_count;
	  if (event.value == 0)
	    {
	      ev1.type = KEYEVENT_RELEASE;
	    }
	  else
	    {
	      ev1.type = KEYEVENT_PRESS;
	    }
	  ev1.key = event.code;
	  event_handler (im, &ev1);
	  /* send key accumulation event */
	  if (keyboard_count == 0 && keyboard_count_max != 0)
	    {
	      if (keyboard_count_max == 4 && keyboard_array_max[KEY_J]
		       && keyboard_array_max[KEY_K] && keyboard_array_max[KEY_L]
		       && keyboard_array_max[KEY_SPACE])
		{
		  if(im!=INPUTMODE_BIG5)
                  {
                    imflag=im;
		    im=INPUTMODE_BIG5;
		    playsound("tobig5");
                  }
		  else
                  {
		    im=imflag;
		    playsound("toenglish");
		  }
		  keyboard_count_max = 0;
		}
	      else if (keyboard_count_max == 4 && keyboard_array_max[KEY_F]
		       && keyboard_array_max[KEY_D] && keyboard_array_max[KEY_S]
		       && keyboard_array_max[KEY_SPACE])
		{
		  if(im!=INPUTMODE_LIU)
                  {
                    imflag=im;
		    im=INPUTMODE_LIU;
		    playsound("tobig5");
                  }
		  else
                  {
		    im=imflag;
		    playsound("toenglish");
		  }
		  keyboard_count_max = 0;
		}
	    }
	  if (keyboard_count == 0 && keyboard_count_max != 0)
	    {
	      memset (&ev1, 0, sizeof (ev1));
	      memcpy (ev1.keyarray, keyboard_array_max,
		      sizeof (ev1.keyarray));
	      ev1.keyarray_count = keyboard_count_max;
	      ev1.type = KEYEVENT_ACCUMULATION;
	      event_handler (im, &ev1);
	      memset (keyboard_array_max, 0, sizeof (keyboard_array_max));
	      keyboard_count_max = 0;
	    }
	}

  return 0;
}
