/*  brlkey
    Copyright(C) 2008 PaulLiu(Ying-Chun Liu). All rights reserved.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include <config.h>
#include <cstdio>
#include <cstdlib>
#include <cstring>
#include <ctime>

#include <termios.h>
#include <unistd.h>
#include <pty.h>
#include <signal.h>
#include <sys/types.h>
#include <sys/wait.h>

#include <linux/input.h>

#include "common.h"

using namespace std;

char transtable1[TRANSTABLE1MAXSIZE][2][100];

int
iso8859_trans1 (const int *keyboard_array, int keyboard_count)
/* search output string in table */
{
  int mykbar[KEYARRAYSIZE], mykbar_count = 0;
  int i;
  for (i = 0; i < TRANSTABLE1MAXSIZE; i++)
    {
      mykbar_count = genarray (transtable1[i][0], mykbar);
      if (keyboard_count == mykbar_count
	  && memcmp (mykbar, keyboard_array, sizeof (mykbar)) == 0)
	{
	  return i;
	}
    }
  return -1;
}

int
readiso8859table1 (void)
/* load translate table */
{
  FILE *file1;
  char filename[1000];
  int i;
  char com1[100], com2[100];
  for (i = 0; i < TRANSTABLE1MAXSIZE; i++)
    {
      transtable1[i][0][0] = '\0';
      transtable1[i][1][0] = '\0';
    }
  strcpy (filename, PKGDATADIR);
  strcat (filename, "/");
  strcat (filename, "iso8859table1.txt");
  file1 = fopen (filename, "rt");
  if (file1 == NULL)
    {
      printf ("Error: cannot open %s\n", filename);
      return -1;
    }
  i = 0;
  while (!feof (file1) && i < TRANSTABLE1MAXSIZE)
    {
      if (fscanf (file1, "%s%s", com1, com2) != 2)
	break;
      strcpy (transtable1[i][0], com1);
      toprintfstring (com2);
      strcpy (transtable1[i][1], com2);
      i++;
    }
  fclose (file1);
  return 0;
}

int
iso8859_event_handler (const struct keyevent *ev)
{
  int now, r1;

  now = (*ev).key;
  switch ((*ev).type)
    {
    case KEYEVENT_RELEASE:
    case KEYEVENT_PRESS:
      if (isnormalkey (now) || (*ev).keyarray[KEY_LEFTALT]
	  || (*ev).keyarray[KEY_RIGHTALT] || (*ev).keyarray[KEY_LEFTCTRL]
	  || (*ev).keyarray[KEY_RIGHTCTRL] || (*ev).keyarray[KEY_LEFTSHIFT] || (*ev).keyarray[KEY_RIGHTSHIFT])
	{
	  struct input_event event = {};
	  event.type = EV_KEY;
	  event.code = now;
	  event.value = (*ev).type == KEYEVENT_PRESS;
	  if (normalhandler(ev) == 0)
	    output_event(&event);
	}
      break;
    case KEYEVENT_ACCUMULATION:
      r1 = iso8859_trans1 ((*ev).keyarray, (*ev).keyarray_count);
      if (r1 >= 0)
	{
	  output_string (transtable1[r1][1]);
	}
      break;
    default:
      break;

    }
  return 0;
}
