/*  brlkey
    Copyright(C) 2008 PaulLiu(Ying-Chun Liu). All rights reserved.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include <sys/types.h>
#include <linux/input.h>
#include <linux/uinput.h>
#include <linux/kd.h>
#include <dirent.h>
#include <stdio.h>
#include <string.h>
#include <fcntl.h>
#include <limits.h>
#include <unistd.h>
#include <stdlib.h>
#include <errno.h>

using namespace std;

static int out, in;

#define INPUT_DIR "/dev/input"

static int
input_in_init(void)
{
    DIR *dir;
    struct dirent *entry;
    
    dir = opendir(INPUT_DIR);
    if (!dir) {
	fprintf(stderr,"Can't list "INPUT_DIR".\n");
	return -1;
    }

    while ((entry = readdir(dir))) {
	char buf[strlen(INPUT_DIR) + 1 + strlen(entry->d_name) + 1];
	unsigned char keys[(KEY_MAX+1)/CHAR_BIT];

	snprintf(buf, sizeof(buf), "%s/%s", INPUT_DIR, entry->d_name);
	in = open(buf, O_RDONLY);
	if (in == -1) {
	  fprintf(stderr, "can't open %s: %s\n", buf, strerror(errno));
	  continue;
	}
	ioctl(in, EVIOCGBIT(EV_KEY, KEY_MAX), &keys);
#define check_key(key) \
	if (!(keys[(key)/CHAR_BIT] & (1 << ((key) % CHAR_BIT)))) { \
	    /* fprintf(stderr,"%s doesn't have "#key".\n", buf); */ \
	    close(in); \
	    in = -1; \
	    continue; \
	}
        check_key(KEY_A);
        check_key(KEY_S);
        check_key(KEY_D);
        check_key(KEY_F);
        check_key(KEY_J);
        check_key(KEY_K);
        check_key(KEY_L);
        check_key(KEY_SEMICOLON);
	/* etc. */
	printf("Using %s\n", buf);
	ioctl(in, EVIOCGRAB, 1);
	return 0;
    }
    fprintf(stderr,"Can't find a suitable input device.\n");
    return -1;
}

static int
input_out_init(void)
{
    struct uinput_user_dev setup = {};
    int key;
    system("modprobe uinput");
    while(1) {
      out = open(INPUT_DIR "/uinput", O_RDWR);
      if (out >= 0) break;
      if (errno != ENOENT) {
        perror("open");
        exit(1);
      }
      sleep(1);
    }
    strncpy(setup.name, "brlkey", sizeof(setup.name));
    if (write(out, &setup, sizeof(setup)) == -1) return -1;
    if (ioctl(out, UI_SET_EVBIT, EV_KEY) == -1) return -1;
    if (ioctl(out, UI_SET_EVBIT, EV_REP) == -1) return -1;
    for (key = 0; key < KEY_MAX; key++)
      if (ioctl(out, UI_SET_KEYBIT, key));
    if (ioctl(out, UI_DEV_CREATE) == -1) return -1;
    if (out == -1) return -1;
    return 0;
}

int
linux_init(void)
{
    if (input_in_init()) return -1;
    /* Out after In to easily avoid grabbing ourselves */
    if (input_out_init()) return -1;
    return 0;
}

int
input_event(struct input_event *event)
{
  return read(in, event, sizeof(*event));
}

int
output_event(const struct input_event *event)
{
  return write(out, event, sizeof(*event));
}

int
output_string(const char *c)
{
  const char *d;
  int console = open("/dev/tty0", O_RDWR);
  for (d = c; *d; d++)
    ioctl(console, TIOCSTI, d);
  close(console);
  return 0;
}

int
getleds(void)
{
  char leds;
  int console = open("/dev/tty0", O_RDWR);
  ioctl(console, KDGETLED, &leds);
  close(console);
  return leds;
}
